package config

import (
	"github.com/kelseyhightower/envconfig"
	"gitlab.schukai.com/oss/minerva/utils"
	"gopkg.in/yaml.v3"
	"os"
	"path"
)

func GetServerPort() string {
	return configuration.Server.Port
}

type Configuration struct {
	//LogLevel string `yaml:"logLevel" envconfig:"LOG_LEVEL"`

	Server struct {
		Port string `yaml:"Port" envconfig:"SERVER_PORT" default:"80"`
	} `yaml:"Server"`

	Paths struct {
		Web string `yaml:"Web" envconfig:"PATH_WEB" default:"/srv/web/"`
	} `yaml:"Paths"`
}

var configuration *Configuration

func GetConfiguration() *Configuration {

	if configuration == nil {
		intiConfiguration()
		readEnv()
	}

	return configuration
}

func InitializeConfigFromFile(filename string) {
	readFile(filename)
}

func intiConfiguration() {

	current, err := os.Getwd()
	if err == nil {
		if readConfig(current) {
			return
		}

	}

	home, err := os.UserHomeDir()
	if err == nil {
		c := path.Join(home, "minerva", ".config")
		if readConfig(c) {
			return
		}
	}

	c := "/etc/minerva/.config"
	if readConfig(c) {
		return
	}

	configuration = &Configuration{}

}

func readConfig(directory string) (found bool) {

	found = false

	if !utils.DirectoryExists(directory) {
		return
	}

	filename := path.Clean(path.Join(directory, "config.yaml"))
	if utils.FileExists(filename) {
		readFile(filename)
		found = true
		return
	}

	return
}

func readFile(filename string) {

	data, err := os.ReadFile(filename)
	if err != nil {
		utils.PrintErrorAndExit(err.Error())
	}

	defer func() {
		if err := recover(); err != nil {
			utils.PrintErrorAndExit("the configuration %s could not be read (%s).", filename, err)
		}
	}()

	c := Configuration{}
	err = yaml.Unmarshal([]byte(data), &c)

	configuration = &c

	if err != nil {
		utils.PrintErrorAndExit(err.Error())
	}
}

func readEnv() {

	err := envconfig.Process("", configuration)
	if err != nil {
		utils.PrintErrorAndExit(err.Error())
	}
}
